//****************************************************************************
//
// NAME: IMU6410plot.c
//
// DATE: March 15, 2011.
//
// PURPOSE: IMU6410 data acquisition and display
//
// Version: 1.0
//
//****************************************************************************
#include "stdafx.h"

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
#include <time.h>
#include <sys/timeb.h>

#pragma warning( disable : 4018 )

#include "wsc.h"
#include "keycode.h"
#include "sayerror.h"

#define     DEBUGON  0

#define		YES				'y'
#define		NO				'n'
#define		LF				0x0a
#define		CR				0x0d
#define     INTELFILELENGTHMAX  9000000
#define     INTELRECORDLENGTH   16

#define		VERBOSE			'v'
#define		QUIET			'q'
#define		INTERACTIVE		'i'
#define     NOTINTERACTIVE	'n'
#define     NORMAL			'n'
#define		VERIFY			'v'
#define		DONTVERIFY		'n'
#define		PROGRAMFLASH	'f'
#define		PROGRAMEEPROM	'e'
#define		RANGESTARTONLY	's'
#define		RANGESTARTEND	'r'
#define		NORANGE			'n'
#define		HANDSHAKEOK		'h'
#define     HANDSHAKEOFF	'o'
#define		UPLOADFLASH		'f'
#define		UPLOADEEPROM	'e'
#define		DISPLAYBAR		1
#define		NOBAR			2
#define     NOCOMCHANNEL    -1

unsigned char fillchar,flasharray[INTELFILELENGTHMAX];
char Temp[120],firstrecord;
int intelrecordlength;
long intelhexfilelength,startaddress,endaddress,filestartaddress,fileendaddress;

struct sfprog {
	char	downloadfilename[200];
	char	uploadfilename[200];
	char	logfilename[200];
	char	mismatchfilename[200];
	char	programmode;
	char	uploadmode;
	char	downloadmode;
	char    handshakemode;
	char	logmode;
	char	mismatchmode;
	char	verifymode;
	char	verbosemode;
	char	rangemode;
	char	targetgetidmode;
	char	targetgetflashtypemode;
	char	interactivemode;
	int		comport;
	int		baudrate;
	int		analog[8];
	unsigned char	fillchar;
    long	startaddress;
	long	endaddress;
}  sfprogstate;


FILE *fpdown,*fpup,*fplog,*fpmatch,*fpconfig;

//***********************************************************
//
//  Utilities
//
//***********************************************************
void DelayUsec(long delay)
{
	delay *= 172;
	while(delay--) ;
}

void DelayMsec(long delay)
{
	while(delay--) DelayUsec(1000);
}

// display error & exit
void SayError(int errorcode,char *Msg)
{
	printf("ERROR: %s: %d\n", Msg, GetLastError() );
//	exit(1);
}

void DisplayHelp()
{
	printf("Command options:\n\n");
	printf("  -d[f|e]filename  - download Hex file to target flash/eeprom using 'filename'\n");
	printf("  -ldfilename - load Intel Hex file using 'filename' - don't program \n");
	printf("  -u[f|e]filename - upload target flash/eeprom contents to 'filename'\n");
	printf("  -lffilename - Log activities to 'filename'\n");
	printf("  -mffilename - Record verify mismatchs to 'filename'\n");
	printf("  -rsssss[,eeeee] - Set upload/download range start and end address\n");
	printf("  -sfxx       - Set fill character - default 0xFF\n");
 	printf("  -v          - Verify download: default no verify\n");
 	printf("  -i          - Identify target\n");
 	printf("  -st         - Update target with host time\n");
 	printf("  -gt         - Get time from target\n");
 	printf("  -ga         - Read eight analog target channels\n");
    printf("  -h[o|f]     - download handshake mode o-on or f-off - default on\n");
	printf("  -cCOMx      - set serial communication channel to COMx\n");
	printf("  -bnnnn      - set buad rate to nnnn (2400-57200 default 38400)\n");
//	printf("  -m[v|n|q]   - set output mode: v-verbose, n-normal, q-quiet\n");
	printf("  -h          - output help menu\n\n");
	printf("  Downloading a hex file automatically sets start and end address for verify.\n");
	printf("  The address range must be set for the upload command to work\n\n");
}

int ErrorCheck(int port,int Code)
{
	if(Code<0)  {
		printf("\n");
		SayError(Code,"Com port maybe wrong or in use by another application\n       Select another com port or clear conflict - COM");
		SioDone(port);
		printf("\n");
		return -1;
		}
	return Code;
}


void flushCRLF(void)
{
	int i;

	while((i = SioGetc(sfprogstate.comport)) < 0)  ;
	printf("%c", i);
//	fprintf(fpup,"%c",i);
	while((i = SioGetc(sfprogstate.comport)) < 0)  ;
	printf("%c", i);
//	fprintf(fpup,"%c",i);
}


void LogEvent(char *event1,char *event2)
{
	char strg1[20],strg2[20];

	_strtime(strg1);
	_strdate(strg2);

	fprintf(fplog,"%s %s: %s%s\n",strg1,strg2,event1,event2);
}
	
int InitSerialPort()
{
	// Pass the key code
	if(SioKeyCode(WSC_KEY_CODE)<0)  {
		printf("ERROR: Bad Key Code!");
		SioDone(sfprogstate.comport);
		exit(1);   
		}

	// Set defaults for all ports. Note 'Port' argument is -1
	ErrorCheck(-1,SioReset(-1,1,1)); // DTR & RTS set at port initialization
	ErrorCheck(-1,SioParms(-1,WSC_NoParity,WSC_OneStopBit,WSC_WordLength8));

	// Reset (initialize) the port
	if(ErrorCheck(sfprogstate.comport,SioReset(sfprogstate.comport,1024,1024))==-1) return -1;
	if(ErrorCheck(sfprogstate.comport,SioBaud(sfprogstate.comport,sfprogstate.baudrate))==-1) return -1;
	return 1;
}

void SetComSerial(void)
{
	SioDone(sfprogstate.comport);
	ErrorCheck(sfprogstate.comport,SioReset(sfprogstate.comport,1024,1024));
	ErrorCheck(sfprogstate.comport,SioBaud(sfprogstate.comport,sfprogstate.baudrate));
}

void  InitIntelHexBuffer(void)
{
	long i;

	for(i=0; i<INTELFILELENGTHMAX; i++)  flasharray[i] = sfprogstate.fillchar;
}

int  LoadIntelFile(void)
{
	char bytehex[3],addresshex[5],firstrecord;
	unsigned char inchar,intelhex[200],crcvalue;
	unsigned int recordaddress,extendedaddress;
	int i,bytecnt,recordtype,recordcnt;
	long totalbytecnt,ilong,maxaddress,baseaddress;

	for(ilong=0; ilong<INTELFILELENGTHMAX; ilong++)  flasharray[ilong] = sfprogstate.fillchar;

	if((fpdown=fopen(sfprogstate.downloadfilename,"r"))==NULL) {
		printf("Can't load %s - file may not exist!\n",sfprogstate.downloadfilename);
		return -1;
		}
	else printf("Load %s\n",sfprogstate.downloadfilename);

	if(sfprogstate.logmode==YES) LogEvent("Load file ",sfprogstate.downloadfilename);

	bytehex[2] = 0x00;
	addresshex[4] = 0x00;
	recordcnt = 1;
	totalbytecnt = 0;
	maxaddress = 0;
	recordaddress = extendedaddress = 0;
	firstrecord = YES;

	while((inchar=getc(fpdown))!=EOF) {
		if(inchar==':') {
			// Decode number of hex bytes on line
			bytehex[0] = getc(fpdown);
			bytehex[1] = getc(fpdown);
			sscanf(bytehex,"%x",&bytecnt);
			// Decode hex record start address
			addresshex[0] = getc(fpdown);
			addresshex[1] = getc(fpdown);
			addresshex[2] = getc(fpdown);
			addresshex[3] = getc(fpdown);
			sscanf(addresshex,"%x",&recordaddress);
			// Decode record type
			bytehex[0] = getc(fpdown);
			bytehex[1] = getc(fpdown);
			sscanf(bytehex,"%x",&recordtype);
			// Decode record data

				//fprintf(fpup,":%.2X%.4X%.2X",bytecnt,baseaddress,recordtype);

			baseaddress = extendedaddress*65536 + recordaddress;

			if(recordtype==0) {
				for(i=0; i<bytecnt; i++) {
					totalbytecnt++;
					bytehex[0] = getc(fpdown);
					bytehex[1] = getc(fpdown);
					sscanf(bytehex,"%x",&intelhex[i]);
					flasharray[baseaddress+i] = intelhex[i];

						//fprintf(fpup,"%.2X",intelhex[i]);

					}
				bytehex[0] = getc(fpdown);
				bytehex[1] = getc(fpdown);
				sscanf(bytehex,"%2x",&crcvalue);

						//fprintf(fpup,"%.2X\n",crcvalue);

				if((baseaddress+bytecnt)>maxaddress)  maxaddress = baseaddress + bytecnt;
				}
			// End of hex file
			else if(recordtype==1) {
				bytehex[0] = getc(fpdown);
				bytehex[1] = getc(fpdown);
				sscanf(bytehex,"%x",&crcvalue);

						//fprintf(fpup,"%.2X\n",crcvalue);

				//printf("Total byte cnt = %d maxaddress = %.6x\n",totalbytecnt,maxaddress);
				intelhexfilelength = maxaddress;
				fileendaddress = maxaddress;
				fclose(fpdown);
				return 1;
				        //fclose(fpup);
				}
			//  Extended address record
			else if(recordtype==4) {
				addresshex[0] = getc(fpdown);
				addresshex[1] = getc(fpdown);
				addresshex[2] = getc(fpdown);
				addresshex[3] = getc(fpdown);
				sscanf(addresshex,"%x",&extendedaddress);
				bytehex[0] = getc(fpdown);
				bytehex[1] = getc(fpdown);
				sscanf(bytehex,"%x",&crcvalue);

				//printf("recordtype=2  extendedaddress = %d\n",extendedaddress);
						//fprintf(fpup,"%.2X\n",crcvalue);

				//printf("Total byte cnt = %d maxaddress = %.4x\n",totalbytecnt,maxaddress);
				}
			if(firstrecord==YES) {
				firstrecord = NO;
				if(recordtype==0)  filestartaddress = extendedaddress*65536 + recordaddress;
				else  filestartaddress = extendedaddress*65536;
				//printf("filestartaddress = %d\n",filestartaddress);
				}
			}
		}
	fileendaddress = maxaddress;
	//printf("fileendaddress = %d\n",fileendaddress);
	fclose(fpdown);
	return 1;
}

void SerialPutc(unsigned char inchar)
{
	SioPutc(sfprogstate.comport,inchar);
//	printf("%c",inchar);
}

void SerialPuts(char *instring)
{
	int i;

	for(i=0; i<strlen(instring); i++)  SioPutc(sfprogstate.comport,instring[i]);
//	for(i=0; i<strlen(instring); i++)  printf("%c",instring[i]);
}

int WaitForCharacter(unsigned char matchchar)
{
	unsigned char inchar;
	int i,delaytime;
	long cycles;
	clock_t start, finish;

	cycles = 100000;

	delaytime = 0;
	inchar = 0;
	while(inchar!=matchchar) {
		start = clock();
		while((i = SioGetc(sfprogstate.comport)) < 0)  ;
		inchar = i;
		if((sfprogstate.verbosemode==VERBOSE)||(sfprogstate.verbosemode==NORMAL))
		            printf("%c",inchar);
		//printf("%c -%x- ",inchar,inchar);
		if(sfprogstate.uploadmode==YES)  fprintf(fpup,"%c",inchar);
		finish = clock();
		if(1000.0*((float)(finish-start)/(float) CLOCKS_PER_SEC)>400.0) {
			if(sfprogstate.logmode==YES)  fprintf(fplog,"break lockup");
			delaytime = 400;
			break;
			}
		}
//	printf("%c -%x- ",inchar,inchar);
	finish = clock();
	//if(sfprogstate.logmode==YES)  fprintf(fplog,"delay in msec=%f",1000.0*((float)(finish-start)/(float) CLOCKS_PER_SEC));
	//if(sfprogstate.logmode==YES)  fprintf(fplog," clocks per sec = %d",CLOCKS_PER_SEC);
	return delaytime;	
}

int WaitForOK(void)
{
	char matchOK[3],inchar;
	int i,delaytime;
	long cycles;
	clock_t start, finish;

	cycles = 100000;
	matchOK[0] = ' ';
	matchOK[1] = ' ';
	matchOK[2] = 0x00;

	delaytime = 0;
	while(strcmp(matchOK,"OK")) {
		start = clock();
		while((i = SioGetc(sfprogstate.comport)) < 0)  ;
		inchar = i;
        matchOK[0] = matchOK[1];
        matchOK[1] = inchar;
		printf("%c",matchOK[1]);
		if(sfprogstate.uploadmode==YES)  fprintf(fpup,"%c",matchOK[1]);
		finish = clock();
		if(1000.0*((float)(finish-start)/(float) CLOCKS_PER_SEC)>400.0) {
			if(sfprogstate.logmode==YES)  fprintf(fplog,"break lockup");
			delaytime = 400;
			break;
			}
		}
	finish = clock();
	if(sfprogstate.logmode==YES)  fprintf(fplog,"delay in msec=%f",1000.0*((float)(finish-start)/(float) CLOCKS_PER_SEC));
	if(sfprogstate.logmode==YES)  fprintf(fplog," clocks per sec = %d",CLOCKS_PER_SEC);
	return delaytime;
}

void SerialGets(char *instrg)
{
	char inchar;
	int strglen,i,j;

	strglen = strlen(instrg);
	for(i=0; i<strglen; i++)  {
		while((j=SioGetc(sfprogstate.comport))<0) ;
		inchar = j;
		if((sfprogstate.verbosemode==VERBOSE)||(sfprogstate.verbosemode==NORMAL))
		    printf("%c",inchar);
		}
}

unsigned char SerialGetc(void)
{
	unsigned char inchar;
	int k;

	while((k=SioGetc(sfprogstate.comport))<0) ;
	inchar = k;
	return inchar;
}

void FlushSerial()
{
	while(!SioGetc(sfprogstate.comport)<0) ;
}

int GetIntelHexFromTarget()
{
	char bytehex[3],addresshex[5],uploadmode,strg1[20],strg2[20];
	unsigned char inchar,intelhex[200],crcvalue;
	unsigned int recordaddress,extendedaddress;
	int i,bytecnt,recordtype,recordcnt;
	long totalbytecnt,maxaddress,mismatches,baseaddress,numtargetbytes;
	long startaddresss,endaddresss;

	uploadmode = NO;
    if((sfprogstate.uploadmode==UPLOADFLASH)||(sfprogstate.uploadmode==UPLOADEEPROM)) {
		uploadmode = YES;
		}
	if((sfprogstate.uploadmode==UPLOADFLASH)||(sfprogstate.uploadmode==UPLOADEEPROM)) {

		if((fpup=fopen(sfprogstate.uploadfilename,"w"))==NULL) return -1;
		if(sfprogstate.logmode==YES) LogEvent("Upload to file ",sfprogstate.uploadfilename);
		//printf("open %s\n",sfprogstate.uploadfilename);
		}
	if(sfprogstate.mismatchmode==YES) {
		if((fpmatch=fopen(sfprogstate.mismatchfilename,"w"))==NULL) return -1;
		}

	bytehex[2] = 0x00;
	addresshex[4] = 0x00;
	recordcnt = 1;
	totalbytecnt = 0;
	maxaddress = 0;
	mismatches = 0;
	baseaddress = extendedaddress = 0;

	if(sfprogstate.rangemode==RANGESTARTEND) {
		numtargetbytes = sfprogstate.endaddress - sfprogstate.startaddress;
		startaddresss = sfprogstate.startaddress;
		endaddresss = sfprogstate.endaddress;
		}
	else {
		numtargetbytes = fileendaddress-filestartaddress;
		startaddresss = filestartaddress;
		endaddresss = fileendaddress;
		}

	if(sfprogstate.logmode==YES) {
		sprintf(strg1,"%x",startaddresss);
		sprintf(strg2,"%x",endaddresss);
		strcat(strg1," to ");
		strcat(strg1,strg2);
		LogEvent("Upload target from ",strg1);
		}

	if(sfprogstate.verifymode==VERIFY) printf("Verify target\n");

	while(totalbytecnt<=numtargetbytes) {
		if(kbhit())	break;
		inchar = SerialGetc();
		if((sfprogstate.verbosemode==VERBOSE)||(sfprogstate.verbosemode==NORMAL))
             printf("%c",inchar);
		if(inchar==':') {
			// Decode number of hex bytes on line
			bytehex[0] = SerialGetc();
			bytehex[1] = SerialGetc();
			sscanf(bytehex,"%x",&bytecnt);
			// Decode hex record start address
			addresshex[0] = SerialGetc();
			addresshex[1] = SerialGetc();
			addresshex[2] = SerialGetc();
			addresshex[3] = SerialGetc();
			sscanf(addresshex,"%x",&recordaddress);
			// Decode record type
			bytehex[0] = SerialGetc();
			bytehex[1] = SerialGetc();
			sscanf(bytehex,"%x",&recordtype);
			// Decode record data

				if(uploadmode==YES)
					fprintf(fpup,":%.2X%.4X%.2X",bytecnt,recordaddress,recordtype);

				if((sfprogstate.verbosemode==VERBOSE)||(sfprogstate.verbosemode==NORMAL))
				    printf("%.2X%.4X%.2X",bytecnt,recordaddress,recordtype);

				baseaddress = extendedaddress*65536 + recordaddress;
				
			if(recordtype==0) {
				for(i=0; i<bytecnt; i++) {
					totalbytecnt++;
					bytehex[0] = SerialGetc();
					bytehex[1] = SerialGetc();
					sscanf(bytehex,"%x",&intelhex[i]);
					if(intelhex[i]!=flasharray[baseaddress+i]) {
						mismatches++;
						if(sfprogstate.mismatchmode==YES) 
							fprintf(fpmatch,"mismatch %d at: %.4x  file=%.2x  target=%.2x\n",
							     mismatches,baseaddress+i,flasharray[baseaddress+i],intelhex[i]);
						}	

					if(uploadmode==YES)
						fprintf(fpup,"%.2X",intelhex[i]);

					if((sfprogstate.verbosemode==VERBOSE)||(sfprogstate.verbosemode==NORMAL))
					    printf("%.2X",intelhex[i]);

					}
				bytehex[0] = SerialGetc();
				bytehex[1] = SerialGetc();
				sscanf(bytehex,"%2x",&crcvalue);

						if(uploadmode==YES)
							fprintf(fpup,"%.2X\n",crcvalue);

						if((sfprogstate.verbosemode==VERBOSE)||(sfprogstate.verbosemode==NORMAL))
						    printf("%.2X",crcvalue);

				if((baseaddress+bytecnt)>maxaddress)  maxaddress = baseaddress + bytecnt;
				//printf(" maxaddress = %d",maxaddress);
				}
			else if(recordtype==1) {
				bytehex[0] = SerialGetc();
				bytehex[1] = SerialGetc();
				sscanf(bytehex,"%x",&crcvalue);

					if(uploadmode==YES)
						fprintf(fpup,"%.2X\n",crcvalue);

					if((sfprogstate.verbosemode==VERBOSE)||(sfprogstate.verbosemode==NORMAL))
					    printf("%.2X",crcvalue);

				//printf("Total byte cnt = %d maxaddress = %.4x\n",totalbytecnt,maxaddress);
				intelhexfilelength = maxaddress;
				if(uploadmode==YES)  fclose(fpup);
				if(sfprogstate.mismatchmode==YES) {
					fprintf(fpmatch,"Total mismatches = %d\n",mismatches);
					fclose(fpmatch);
					}
				WaitForCharacter('-');
				if(sfprogstate.verifymode==VERIFY)  {
					if(mismatches==0)  printf("\nVerified - no mismatches","");
					else printf("\nErrors in verify - mismatches = %d",mismatches);
					}
				if(sfprogstate.logmode==YES) {
					if(mismatches==0)  LogEvent("Verified - no mismatches","");
					else {
						sprintf(strg1,"%d",mismatches);
						LogEvent("Errors in verify - mismatches = ",strg1);
						}
					}
				return 1;
				}
			else if(recordtype==4) {
				bytehex[0] = SerialGetc();
				bytehex[1] = SerialGetc();
				sscanf(bytehex,"%x",&crcvalue);

				extendedaddress = recordaddress;

					if(uploadmode==YES)
						fprintf(fpup,"%.2X\n",crcvalue);
					printf("%.2X",crcvalue);

				//printf("Total byte cnt = %d maxaddress = %.4x\n",totalbytecnt,maxaddress);
				}
			}
		}
	WaitForCharacter('-');

	if(sfprogstate.verifymode==VERIFY)  {
		if(mismatches==0)  printf("\nVerified - no mismatches","");
		else printf("\nErrors in verify - mismatches = %d",mismatches);
		}

	if(uploadmode==YES) fclose(fpup);

	if(sfprogstate.mismatchmode==YES) {
		fprintf(fpmatch,"Total mismatches = %d\n",mismatches);
		fclose(fpmatch);
		}

	if(sfprogstate.logmode==YES) {
		if(mismatches==0)  LogEvent("Verified - no mismatches","");
		else {
			sprintf(strg1,"%d",mismatches);
			LogEvent("Errors in verify - mismatches = ",strg1);
			}
		}
	//printf("exiting get intel hex\n");
	return 1;
}

// Read back to download and verify against the load image
// Write difference to the logdif file
int VerifyLoad(void)
{
	char outstrg[50],readstrg[100];

	if(sfprogstate.uploadmode==UPLOADFLASH)  {
		if(sfprogstate.logmode==YES) LogEvent("Upload from target Flash","");
		SerialPuts("df");
		SerialGets("df\rEnter start address and length: ");
		}
	else if(sfprogstate.uploadmode==UPLOADEEPROM) {
		if(sfprogstate.logmode==YES) LogEvent("Upload from target EEPROM","");
		SerialPuts("de");
		SerialGets("de\rEnter start address and length: ");
		}
	else if(sfprogstate.programmode==PROGRAMFLASH)  {
		if(sfprogstate.logmode==YES) LogEvent("Upload from target Flash","");
		SerialPuts("df");
		SerialGets("df\rEnter start address and length: ");
		}
	else if(sfprogstate.programmode==PROGRAMEEPROM) {
		if(sfprogstate.logmode==YES) LogEvent("Upload from target EEPROM","");
		SerialPuts("de");
		SerialGets("de\rEnter start address and length: ");
		}

	if(sfprogstate.logmode==YES) {
		if(sfprogstate.verifymode==VERIFY)  LogEvent("Verify Mode ON","");
		else  LogEvent("Verify Mode OFF","");
		}

//	printf("df\n");
//	printf("df\nEnter start address and length: ");
	if(sfprogstate.rangemode==RANGESTARTEND)  sprintf(outstrg,"%.4x",sfprogstate.startaddress);
	else sprintf(outstrg,"%.4x",filestartaddress);
	//printf("Startaddress=%s\n",outstrg);
	strcpy(readstrg,outstrg);
	strcat(readstrg," ");
	if(sfprogstate.rangemode==RANGESTARTEND)  
		  sprintf(outstrg,"%.4x",sfprogstate.endaddress-sfprogstate.startaddress);
	else  sprintf(outstrg,"%.4x",fileendaddress-filestartaddress);
	//printf("length=%s\n",outstrg);
	strcat(readstrg,outstrg);
	strcat(readstrg,"\r ");
//	printf("%s",readstrg);
	SerialPuts(readstrg);
//	printf("%s",readstrg);
	SerialGets(readstrg);

	// Read the return data
	if(GetIntelHexFromTarget()) return 1;
	else return 0;
}

int  SelectTargetProgramMode()
{
	if(sfprogstate.programmode==PROGRAMFLASH) {
		SerialPuts("pf");
		if(WaitForCharacter('-')>350) return 0;
		}
	else if(sfprogstate.programmode==PROGRAMEEPROM)  {
		SerialPuts("pe");
		if(WaitForCharacter('-')>350) return 0;
		}
	return 1;
}


int GetPortId(char *instrg)
{
	//printf(" instrg=%s\n",instrg);
	if(!stricmp("com1",instrg))       return 0;
	else if(!stricmp("com2",instrg))  return 1;
	else if(!stricmp("com3",instrg))  return 2;
	else if(!stricmp("com4",instrg))  return 3;
	else if(!stricmp("com5",instrg))  return 4;
	else if(!stricmp("com6",instrg))  return 5;
	else if(!stricmp("com7",instrg))  return 6;
	else if(!stricmp("com8",instrg))  return 7;
	else if(!stricmp("com9",instrg))  return 8;
	else if(!stricmp("com10",instrg))  return 9;
	else if(!stricmp("com11",instrg))  return 10;
	else {
		printf("Unrecognized com channel: %s\n",instrg);
		return 0;
		}
}

long GetBaudRate(char *instrg)
{
	if(!strcmp("2400",instrg))  return WSC_Baud2400;
	else if(!strcmp("4800",instrg))  return WSC_Baud4800;
	else if(!strcmp("9600",instrg))  return WSC_Baud9600;
	else if(!strcmp("19200",instrg))  return WSC_Baud19200;
	else if(!strcmp("38400",instrg))  return WSC_Baud38400;
	else if(!strcmp("57200",instrg))  return WSC_Baud57600;
	else if(!strcmp("115200",instrg))  return WSC_Baud115200;
	else {
		printf("Unrecognized baud rate\n");
		return WSC_Baud38400;
		}
}

int ParseConfigFile()
{
	char token[100];

	if((fpconfig=fopen("imu6410config.txt","r"))==NULL)  {
		//printf("no config file\n");
		return -1;
		}

	while(fscanf(fpconfig,"%s",token)!=EOF) {
		//printf("token = %s\n",token);
		if(!stricmp("baudrate",token))  {
			fscanf(fpconfig,"%s",token);
		//printf(" token = %s\n",token);
			sfprogstate.baudrate = GetBaudRate(token);
			//printf("new baud rate\n");
			}
		else if(!stricmp("comchannel",token))  {
			fscanf(fpconfig,"%s",token);
			sfprogstate.comport = GetPortId(token);
		//printf(" token = %s\n",token);
			//printf("new com id\n");
			}
		}
	fclose(fpconfig);
	return 1;
			
}

void InitSfProg()
{
	sfprogstate.programmode = PROGRAMFLASH;
//	sfprogstate.baudrate = WSC_Baud9600;
	sfprogstate.baudrate = WSC_Baud38400;
	sfprogstate.comport = COM2;
//	sfprogstate.comport = COM1;
	sfprogstate.downloadmode = NO;
	sfprogstate.fillchar = 0xFF;
	sfprogstate.interactivemode = NO;
	sfprogstate.handshakemode = HANDSHAKEOK;
	sfprogstate.logmode = NO;
	sfprogstate.mismatchmode = NO;
	sfprogstate.targetgetflashtypemode = NO;
	sfprogstate.targetgetidmode = NO;
	sfprogstate.uploadmode = NO;
	sfprogstate.verbosemode = NORMAL;
	sfprogstate.verifymode = DONTVERIFY;
	sfprogstate.startaddress = 0;
	sfprogstate.endaddress = 0;    // Should check flash size to set this parameter
	filestartaddress = 0;
	fileendaddress = 0;
	firstrecord = YES;
	sfprogstate.rangemode = NORANGE;

	ParseConfigFile();
	
	strcpy(sfprogstate.downloadfilename,"");
	strcpy(sfprogstate.uploadfilename,"");
	strcpy(sfprogstate.logfilename,"");
	strcpy(sfprogstate.mismatchfilename,"");
}

void ResetSfProgState()
{
	sfprogstate.programmode = PROGRAMFLASH;
	sfprogstate.downloadmode = NO;
	sfprogstate.fillchar = 0xFF;
	sfprogstate.interactivemode = NO;
	sfprogstate.logmode = NO;
	sfprogstate.mismatchmode = NO;
	sfprogstate.targetgetflashtypemode = NO;
	sfprogstate.targetgetidmode = NO;
	sfprogstate.uploadmode = NO;
	sfprogstate.verbosemode = NORMAL;
	sfprogstate.verifymode = DONTVERIFY;
	sfprogstate.startaddress = 0;
	sfprogstate.endaddress = 0;    // Should check flash size to set this parameter
	filestartaddress = 0;
	fileendaddress = 0;
	firstrecord = YES;
	sfprogstate.rangemode = NORANGE;

//	ParseConfigFile();
	
	strcpy(sfprogstate.downloadfilename,"");
	strcpy(sfprogstate.uploadfilename,"");
	strcpy(sfprogstate.logfilename,"");
	strcpy(sfprogstate.mismatchfilename,"");

}

long GetStartAddress(char *instrg)
{
	char outstrg[20];
	int i;
	long address;

	i =0;
	while((instrg[i]!=',')&&(instrg[i]!=0x00)) {
		outstrg[i] = instrg[i];
		i++;
		}
	outstrg[i] = 0;
	sscanf(outstrg,"%x",&address);
	return address;
}
	
long GetEndAddress(char *instrg)
{
	char outstrg[20];
	int i,j;
	long address;

	i = j = 0;
	// Skip start address
	while((instrg[i]!=',')&&(instrg[i]!=0x00))  i++;

	// If no second address return 0
	if(instrg[i] == 0x00)  return 0;
	
	i++;
	while(instrg[i]!=0x00) {
		outstrg[j++] = instrg[i];
		i++;
		}
	outstrg[i] = 0;
	sscanf(outstrg,"%x",&address);
	return address;

}

void SignonMessage()
{
	printf("Serial Flash Download Program V1.11\n");
	printf("(c) Copyright 2006-2007, SOC Robotics, Inc.\n");
	printf("Type 'e' to exit, 'p' to pause, 'h' for help)\n");
}

int  ProgramFlashEEPROM()
{
	char lastrecord,firstrecord,outputextendedaddressrecord,outstrg[200],strg1[20],strg2[20];
	unsigned char crcvalue,recordtype;
	long extendedaddress,recordaddress;
	int j,c,bytesonline;
	long startaddress,endaddress;

	if(sfprogstate.programmode==PROGRAMFLASH) {
		SerialPuts("pf");
		WaitForCharacter('-');
		//printf("Program Flash\n");
		}
	else if(sfprogstate.programmode==PROGRAMEEPROM)  {
		SerialPuts("pe");
		WaitForCharacter('-');
//		printf("Program EEPROM\n");
		}

	if(sfprogstate.rangemode==RANGESTARTEND) {
		startaddress = sfprogstate.startaddress;
		endaddress = sfprogstate.endaddress;
		}
	else {
		startaddress = filestartaddress;
		endaddress = fileendaddress;
		}

	if(sfprogstate.logmode==YES)  {
		sprintf(strg1,"%x",startaddress);
		sprintf(strg2,"%x",endaddress);
		strcat(strg1," to ");
		strcat(strg1,strg2);
		LogEvent("Flash target from ",strg1);
		}

	//printf("Startaddress=%d endaddress=%d\n",startaddress,endaddress);
	extendedaddress = startaddress/65536;
	recordaddress = startaddress - extendedaddress*65536;
	//printf("Extendedaddress=%d  recordaddress=%x\n",extendedaddress,recordaddress);

	firstrecord = YES;
	lastrecord = NO;
	recordtype = 0;
	crcvalue = 0;
	bytesonline = intelrecordlength = 16;
	outputextendedaddressrecord = NO;

	printf("Program target\n");

	while((recordaddress+extendedaddress*65536)<endaddress)  {
		// fetch character
		//printf("+");
		if(kbhit())	{
			c = getch();
			if(c=='p') {
				fflush(stdin) ;   // clear stdin's buffer
				printf("\nPause Serial Flash load - press 'r' to resume: ");
				scanf("%c", &c);
				}
			if(c=='e')	 {// restore COM port status & exit
				SerialPuts(":00000001FF\n");
				WaitForCharacter('-');
				return 1;
				}
			}

		// Generate IntelHex32 output 
		//   Output record 2 when 65536 boundary traversed
		//   : ll aaaa rr dd dd dd dd ... cc

		// See if first record and extended address is non zero
		bytesonline = 0;
		if((firstrecord==YES)&&(extendedaddress>0)) {
			firstrecord = NO;
			outputextendedaddressrecord = YES;
			}
		// See if 65536 boundary traversed
		else if((recordaddress+intelrecordlength)>65536) {
			bytesonline = 65536 - recordaddress;
			outputextendedaddressrecord = YES;
			recordaddress = 0;
			extendedaddress++;
			}
		// See if last record
		else if((endaddress-(recordaddress+extendedaddress*65536))<intelrecordlength) {
			//printf("Last record\n");
			bytesonline = endaddress-(recordaddress+extendedaddress*65536);
			lastrecord = YES;
			//printf("Last record - length = %d\n",intelrecordlength);
			}
		else bytesonline = intelrecordlength;

		if(bytesonline>0) {
			// Start intel hex record
			SerialPutc(':');  

			// Output record length
			sprintf(outstrg,"%.2x",bytesonline);
			SerialPuts(outstrg);  
			// Output start address
			sprintf(outstrg,"%.4x",recordaddress);
			SerialPuts(outstrg); 
			// Output recordtype
			sprintf(outstrg,"%.2x",recordtype);
			SerialPuts(outstrg); 
			// Output data 
			for(j=0; j<bytesonline; j++) {
				sprintf(outstrg,"%.2x",flasharray[recordaddress+extendedaddress*65536+j]);
				SerialPuts(outstrg);  
				}

			// Output crc - tbd
			sprintf(outstrg,"%.2x",intelrecordlength);
			SerialPuts("00");  
			SerialPuts("\n");
			WaitForCharacter('-');
			recordaddress += intelrecordlength;
			//intelfileptr += intelrecordlength;
			}

		// Output IntelHex record type 2
		if(outputextendedaddressrecord==YES) {
			// Output intel extended hex record 2
			sprintf(outstrg,"%.4x",extendedaddress);
			//printf("**:02000002%sFF\n",outstrg);
			SerialPuts(":02000004");  
			SerialPuts(outstrg);  
			SerialPuts("FF\n");
			WaitForCharacter('-');
			outputextendedaddressrecord = NO;
			}

		if(lastrecord==YES)  {
			SerialPuts(":00000001FF\n");
			//printf(" end record \n");
			WaitForCharacter('-');
			}
		} 
	return 1;
}

void GetTargetId()
{
	SerialPuts("v");
	WaitForCharacter('-');
	WaitForCharacter('-');
	WaitForCharacter('-');
	WaitForCharacter('-');
}

void SetTime()
{
	char tmpbuf[128];
	int i,j;

	SerialPuts("t");
	for(i=0; i<5; i++) WaitForCharacter(':');
     _strtime(tmpbuf);
    printf("%s\n",tmpbuf);
	SerialPuts(tmpbuf);
	SerialPuts("\r");
	WaitForCharacter('-');
}

void SerialGetcQuiet(int no)
{
	int i;

	for(i=0; i<no; i++)  SerialGetc();
}

void GetTime()
{
	char timebuf[30],inchar;
	int i;

	SerialPuts("t");
	inchar = ' ';
	while(inchar!='=') {
		inchar = SerialGetc();
		//putchar(inchar);
		}
	timebuf[0] = ' ';
	i = 0;
	timebuf[i++] = SerialGetc();
	while(timebuf[i-1]!=':') {
		timebuf[i++] = SerialGetc();
		//putchar(timebuf[i-1]);
		}	
	timebuf[i] = 0;
	printf("IMU6410 time=%s",timebuf);
	//SerialGetcQuiet(2);
	SerialPuts("\r");
	WaitForCharacter('-');
}

void GetAnalog()
{
	char instrg[200],inchar;
	int i,j;

	inchar = ' ';
	SerialPuts("a");
	//printf("a\n");
	j = 0;
	while(inchar!='-') {
		inchar = SerialGetc();
		instrg[j++] = inchar;
		}
	instrg[j] = 0;
	//printf("instrg=%s\n",instrg);
	for(i=0; i<strlen(instrg)-23; i++) instrg[i] = instrg[23+i];
	instrg[strlen(instrg)-22] = 0;
	//printf("instrg=%s\n",instrg);
	sscanf(instrg,"%d %d %d %d %d %d %d %d",&sfprogstate.analog[0],&sfprogstate.analog[1],
											&sfprogstate.analog[2],&sfprogstate.analog[3],
											&sfprogstate.analog[4],&sfprogstate.analog[5],
											&sfprogstate.analog[6],&sfprogstate.analog[7]);
}

int ExpandBaudRate(int buadrate)
{
	if(buadrate==0) return 110;
	else if(buadrate==1) return 300;
	else if(buadrate==2) return 1200;
	else if(buadrate==3) return 2400;
	else if(buadrate==4) return 4800;
	else if(buadrate==5) return 9600;
	else if(buadrate==6) return 19200;
	else if(buadrate==7) return 38400;
	else if(buadrate==8) return 57200;
	else if(buadrate==9) return 115200;
	else return 96;
}

void plot_rawdatalogged_file()
{

    FILE * acc_pipe = _popen("gnuplot -persist","w");
    FILE * gyr_pipe = _popen("gnuplot -persist","w");
    FILE * mag_pipe = _popen("gnuplot -persist","w");


    fprintf(acc_pipe, "set term win\n");
    fprintf(acc_pipe, "set yrange [-3.000:3.000]\n");
    fprintf(acc_pipe, "set title 'IMU6410 Accelerometer Data'\n");
    fprintf(acc_pipe, "set xlabel 'Samples'\n");
    fprintf(acc_pipe, "set ylabel 'Amplitude (g)'\n");
    fprintf(acc_pipe, "set ytics 1 \n");
    fprintf(acc_pipe, "set mytics 10 \n");
    fprintf(acc_pipe, "set grid \n");
    fprintf(acc_pipe, "plot 'imu6410log.txt' using 3 with lines ti 'Accx' \n");
    fprintf(acc_pipe, "replot 'imu6410log.txt' using 4 with lines ti 'Accy' \n");
    fprintf(acc_pipe, "replot 'imu6410log.txt' using 5 with lines ti 'Accz'\n");
    fprintf(acc_pipe, "set term png\n");
    fprintf(acc_pipe, "set output \"acc.png\"\n");
    fprintf(acc_pipe, "replot\n");

    fprintf(gyr_pipe, "set win\n");
    fprintf(gyr_pipe, "set yrange [-300:300]\n");
    fprintf(gyr_pipe, "set title 'IMU6410 Gyroscope Data'\n");
    fprintf(gyr_pipe, "set xlabel 'Samples'\n");
    fprintf(gyr_pipe, "set ylabel 'Amplitude (deg/sec)'\n");
    fprintf(gyr_pipe, "set grid \n");
    fprintf(gyr_pipe, "plot 'imu6410log.txt' using 6 with lines ti 'Angular Velocity (x)' \n");
    fprintf(gyr_pipe, "replot 'imu6410log.txt' using 7 with lines ti 'Angular Velocity (y)' \n");
    fprintf(gyr_pipe, "replot 'imu6410log.txt' using 8 with lines ti 'Angular Velocity (z)'\n");
    fprintf(gyr_pipe, "set term png\n");
    fprintf(gyr_pipe, "set output \"gyr.png\"\n");
    fprintf(gyr_pipe, "replot\n");

    fprintf(gyr_pipe, "set term win\n");
    fprintf(gyr_pipe, "set yrange [-3:3]\n");
    fprintf(mag_pipe, "set yrange [-1:1]\n");
    fprintf(mag_pipe, "set title 'IMU6410 Magnetometer Data'\n");
    fprintf(mag_pipe, "set xlabel 'Samples'\n");
    fprintf(mag_pipe, "set ylabel 'Amplitude (deg/sec)'\n");
    fprintf(mag_pipe, "set grid \n");
    fprintf(mag_pipe, "plot 'imu6410log.txt' using 9 with lines ti 'Mag Field (x)' \n");
    fprintf(mag_pipe, "replot 'imu6410log.txt' using 10 with lines ti 'Mag Field (y)' \n");
    fprintf(mag_pipe, "replot 'imu6410log.txt' using 11 with lines ti 'Mag Field (z)'\n");
    fprintf(mag_pipe, "set term png\n");
    fprintf(mag_pipe, "set output \"mag.png\"\n");
    fprintf(mag_pipe, "replot\n");

    _pclose(acc_pipe);
    _pclose(gyr_pipe);
    _pclose(mag_pipe);
}

// Log and graph MEMs data
void LogMEMData()
{
	char inputbuf[250];
	int i;
	long loopcnt=0,minorloopcnt=0;

    FILE * acc_pipe = _popen("gnuplot -persist","w");
    FILE * gyr_pipe = _popen("gnuplot -persist","w");
    FILE * mag_pipe = _popen("gnuplot -persist","w");

    fprintf(acc_pipe, "set term win\n");
    fprintf(acc_pipe, "set yrange [-3.000:3.000]\n");
    fprintf(acc_pipe, "set title 'IMU6410 Accelerometer Data'\n");
    fprintf(acc_pipe, "set xlabel 'Samples'\n");
    fprintf(acc_pipe, "set ylabel 'Amplitude (g)'\n");
    fprintf(acc_pipe, "set ytics 1 \n");
    fprintf(acc_pipe, "set mytics 10 \n");
    fprintf(acc_pipe, "set grid \n");
    fprintf(acc_pipe, "plot 'imu6410log.txt' using 4 with lines ti 'Accx' \n");
    fprintf(acc_pipe, "replot 'imu6410log.txt' using 5 with lines ti 'Accy' \n");
    fprintf(acc_pipe, "replot 'imu6410log.txt' using 6 with lines ti 'Accz'\n");
    //fprintf(acc_pipe, "set term png\n");
    //fprintf(acc_pipe, "set output \"acc.png\"\n");
    //fprintf(acc_pipe, "replot\n");

    fprintf(gyr_pipe, "set win\n");
    fprintf(gyr_pipe, "set yrange [-300:300]\n");
    fprintf(gyr_pipe, "set title 'IMU6410 Gyroscope Data'\n");
    fprintf(gyr_pipe, "set xlabel 'Samples'\n");
    fprintf(gyr_pipe, "set ylabel 'Amplitude (deg/sec)'\n");
    fprintf(gyr_pipe, "set grid \n");
    //fprintf(gyr_pipe, "plot 'imu6410log.txt' using 6 with lines ti 'Angular Velocity (x)' \n");
    //fprintf(gyr_pipe, "replot 'imu6410log.txt' using 7 with lines ti 'Angular Velocity (y)' \n");
    //fprintf(gyr_pipe, "replot 'imu6410log.txt' using 8 with lines ti 'Angular Velocity (z)'\n");
    //fprintf(gyr_pipe, "set term png\n");
    //fprintf(gyr_pipe, "set output \"gyr.png\"\n");
    //fprintf(gyr_pipe, "replot\n");

    fprintf(mag_pipe, "set term win\n");
    fprintf(mag_pipe, "set yrange [-1:1]\n");
    fprintf(mag_pipe, "set title 'IMU6410 Magnetometer Data'\n");
    fprintf(mag_pipe, "set xlabel 'Samples'\n");
    fprintf(mag_pipe, "set ylabel 'Amplitude (deg/sec)'\n");
    fprintf(mag_pipe, "set grid \n");
    //fprintf(mag_pipe, "plot 'imu6410log.txt' using 9 with lines ti 'Mag Field (x)' \n");
    //fprintf(mag_pipe, "replot 'imu6410log.txt' using 10 with lines ti 'Mag Field (y)' \n");
    //fprintf(mag_pipe, "replot 'imu6410log.txt' using 11 with lines ti 'Mag Field (z)'\n");
    //fprintf(mag_pipe, "set term png\n");
    //fprintf(mag_pipe, "set output \"mag.png\"\n");
    //fprintf(mag_pipe, "replot\n");


	if((fplog=fopen("imu6410log.txt","w"))==NULL)  {
		printf("Can't open log file foo.txt\n");
		}
	SerialPuts("9t");
	while(!kbhit()) {
		i = 0;
		while((inputbuf[i++]=SerialGetc())!='\r') ;
		inputbuf[i] = 0;
		printf("%s\n",inputbuf);
		if(loopcnt++>3) fprintf(fplog,"%d %s\n",loopcnt-2,inputbuf);
		minorloopcnt++;

		// Setup real time gnuplot display
		if((loopcnt>200)&&(minorloopcnt>20)) {
            fprintf(acc_pipe, "set xrange [%d:%d]\n",loopcnt-199,loopcnt);
            fprintf(acc_pipe, "plot 'imu6410log.txt' using 4 with lines ti 'Accx' \n");
            fprintf(acc_pipe, "replot 'imu6410log.txt' using 5 with lines ti 'Accy' \n");
            fprintf(acc_pipe, "replot 'imu6410log.txt' using 6 with lines ti 'Accz'\n");
			minorloopcnt = 0;
			}
		}
	SerialPuts("e");
	fclose(fplog);
	FlushSerial();
	//plot_rawdatalogged_file();
    _pclose(acc_pipe);
    _pclose(gyr_pipe);
    _pclose(mag_pipe);
}

int ParseToken(char *instrg)
{
	char tempstrg[200],outfilename[200],infilename[200],inchar;
	int  i,returnstate;
	long loopcnt;

	returnstate = DISPLAYBAR;
	if(!strnicmp("c",instrg,1)) {
		strcpy(tempstrg,instrg);
		for(i=0; i<strlen(tempstrg)-1; i++)  tempstrg[i] = tempstrg[i+1];
		tempstrg[i] = 0x00;
		sfprogstate.comport = GetPortId(tempstrg);
		SetComSerial();
		printf("Com channel set to COM%d,%d,N,1\n",
			   sfprogstate.comport+1,ExpandBaudRate(sfprogstate.baudrate));
		if(sfprogstate.logmode==YES) LogEvent("Change com port to ",tempstrg);
		if(DEBUGON) printf("Port = %d\n",sfprogstate.comport);
		}
	else if(!strnicmp("b",instrg,1)) {
		strcpy(tempstrg,instrg);
		for(i=0; i<strlen(instrg)-1; i++)  tempstrg[i] = tempstrg[i+1];
		tempstrg[i] = 0x00;
		sfprogstate.baudrate = GetBaudRate(tempstrg);
		if(sfprogstate.logmode==YES) LogEvent("Change baud rate to ",tempstrg);
		SetComSerial();
		if(DEBUGON) printf("Baud rate = %d\n",sfprogstate.baudrate);
		}
	else if(!strnicmp("df",instrg,2)) {
		sfprogstate.programmode = PROGRAMFLASH;
		strcpy(tempstrg,instrg);
		for(i=0; i<strlen(instrg)-2; i++)  tempstrg[i] = tempstrg[i+2];
		tempstrg[i] = 0x00;
		strcpy(infilename,tempstrg);
		strcpy(sfprogstate.downloadfilename,tempstrg);
		sfprogstate.downloadmode = YES;
		LoadIntelFile();
		ProgramFlashEEPROM();
		if(sfprogstate.verifymode==VERIFY)  VerifyLoad();
		ResetSfProgState();
		if(DEBUGON) printf("Download filename=%s\n",sfprogstate.downloadfilename);
		}
	else if(!strnicmp("de",instrg,2)) {
		sfprogstate.programmode = PROGRAMEEPROM;
		strcpy(tempstrg,instrg);
		for(i=0; i<strlen(instrg)-2; i++)  tempstrg[i] = tempstrg[i+2];
		tempstrg[i] = 0x00;
		strcpy(infilename,tempstrg);
		strcpy(sfprogstate.downloadfilename,tempstrg);
		sfprogstate.downloadmode = YES;
		LoadIntelFile();
		ProgramFlashEEPROM();
		if(sfprogstate.verifymode==VERIFY)  VerifyLoad();
		ResetSfProgState();
		if(DEBUGON) printf("Download filename=%s\n",sfprogstate.downloadfilename);
		}
	else if(!strnicmp("ld",instrg,2)) {
		strcpy(tempstrg,instrg);
		for(i=0; i<strlen(instrg)-2; i++)  tempstrg[i] = tempstrg[i+2];
		tempstrg[i] = 0x00;
		strcpy(infilename,tempstrg);
		strcpy(sfprogstate.downloadfilename,tempstrg);
		sfprogstate.downloadmode = NO;
		LoadIntelFile();
		if(sfprogstate.verifymode==VERIFY) {
			VerifyLoad();
			ResetSfProgState();
			}
		if(DEBUGON) printf("Load filename=%s\n",sfprogstate.downloadfilename);
		}
	else if(!strnicmp("uf",instrg,2)) {
		strcpy(tempstrg,instrg);
		for(i=0; i<strlen(instrg)-2; i++)  tempstrg[i] = tempstrg[i+2];
		tempstrg[i] = 0x00;
		strcpy(outfilename,tempstrg);
		strcpy(sfprogstate.uploadfilename,tempstrg);
		sfprogstate.uploadmode = UPLOADFLASH;
		VerifyLoad();
		returnstate = NOBAR;
		ResetSfProgState();
		if(DEBUGON) printf("Upload filename=%s\n",sfprogstate.uploadfilename);
		}
	else if(!strnicmp("ue",instrg,2)) {
		strcpy(tempstrg,instrg);
		for(i=0; i<strlen(instrg)-2; i++)  tempstrg[i] = tempstrg[i+2];
		tempstrg[i] = 0x00;
		strcpy(outfilename,tempstrg);
		strcpy(sfprogstate.uploadfilename,tempstrg);
		sfprogstate.uploadmode = UPLOADEEPROM;
		VerifyLoad();
		returnstate = NOBAR;
		ResetSfProgState();
		if(DEBUGON) printf("Upload filename=%s\n",sfprogstate.uploadfilename);
		}
	else if(!strnicmp("lf",instrg,2)) {
		strcpy(tempstrg,instrg);
		for(i=0; i<strlen(instrg)-2; i++)  tempstrg[i] = tempstrg[i+2];
		tempstrg[i] = 0x00;
		strcpy(sfprogstate.logfilename,tempstrg);
		sfprogstate.logmode = YES;
		fplog = fopen(sfprogstate.logfilename,"w");
		LogEvent("Start log","");
		if(DEBUGON) printf("Log filename=%s\n",sfprogstate.logfilename);
		}
	else if(!strnicmp("mf",instrg,2)) {
		strcpy(tempstrg,instrg);
		for(i=0; i<strlen(instrg)-2; i++)  tempstrg[i] = tempstrg[i+2];
		tempstrg[i] = 0x00;
		strcpy(sfprogstate.mismatchfilename,tempstrg);
		sfprogstate.mismatchmode = YES;
		if(DEBUGON) printf("Mismatch filename=%s\n",sfprogstate.mismatchfilename);
		}
	else if(!strnicmp("r",instrg,1)) {
		strcpy(tempstrg,instrg);
		for(i=0; i<strlen(instrg)-1; i++)  tempstrg[i] = tempstrg[i+1];
		tempstrg[i] = 0x00;
		sfprogstate.startaddress = GetStartAddress(tempstrg);
		sfprogstate.endaddress = GetEndAddress(tempstrg);
		if(sfprogstate.startaddress > sfprogstate.endaddress) 
			  sfprogstate.endaddress = sfprogstate.startaddress;
		if(sfprogstate.endaddress==0) sfprogstate.rangemode = RANGESTARTONLY; 
		else sfprogstate.rangemode = RANGESTARTEND; 
		if((sfprogstate.verbosemode==VERBOSE)||(sfprogstate.verbosemode==NORMAL))
		    printf("Start/End address and length=%x %x %d bytes",
			    sfprogstate.startaddress,sfprogstate.endaddress,
			    sfprogstate.endaddress-sfprogstate.startaddress);
		}
	else if(!strnicmp("sf",instrg,2)) {
		strcpy(tempstrg,instrg);
		for(i=0; i<strlen(instrg)-2; i++)  tempstrg[i] = tempstrg[i+2];
		tempstrg[i] = 0x00;
		sscanf(tempstrg,"%x",&sfprogstate.fillchar);
		if(DEBUGON) printf("Fill char=%x\n",sfprogstate.fillchar);
		}
	else if(!strnicmp("l",instrg,2)) {
		LogMEMData();
		if(DEBUGON) printf("Log MEMs data\n");
		}
	else if(!strnicmp("st",instrg,2)) {
		SetTime();
		if(DEBUGON) printf("Set time\n");
		}
	else if(!strnicmp("gt",instrg,2)) {
		GetTime();
		if(DEBUGON) printf("Get time\n");
		}
	else if(!strnicmp("gc",instrg,2)) {
		//_strtime(tempstrg);
		//printf("%s\n",tempstrg);
		//loopcnt = 0;
		while(!kbhit())	{
			GetAnalog();
			for(i=0; i<8; i++) printf("%d ",sfprogstate.analog[i]);
			printf("             \r");
			//loopcnt++;
			}
		inchar = getch();
		//_strtime(tempstrg);
		//printf("\n%s ",tempstrg);
		//printf("loop cnt=%d\n",loopcnt);
		if(DEBUGON) printf("Set time\n");
		}
	else if(!strnicmp("ga",instrg,2)) {
		GetAnalog();
		for(i=0; i<8; i++) printf("%d ",sfprogstate.analog[i]);
		if(DEBUGON) printf("Set time\n");
		}
	else if(!strnicmp("i",instrg,1)) {
		sfprogstate.targetgetidmode = YES;
		GetTargetId();
		sfprogstate.targetgetidmode = NO;
		returnstate = NOBAR;
		//if(DEBUGON) printf("Identify target\n");
		}
	else if(!strnicmp("gf",instrg,2)) {
		sfprogstate.targetgetflashtypemode = YES;
		if(DEBUGON) printf("Get target flash type\n");
		}
	else if(!strnicmp("hf",instrg,2)) {
		sfprogstate.handshakemode = HANDSHAKEOFF;
		if(DEBUGON) printf("Get target flash type\n");
		}
	else if(!strnicmp("ho",instrg,2)) {
		sfprogstate.handshakemode = HANDSHAKEOK;
		if(DEBUGON) printf("Get target flash type\n");
		}
	else if(!strnicmp("v",instrg,2)) {
		sfprogstate.verifymode = VERIFY;
		//VerifyLoad();
		if(DEBUGON) printf("Verify load\n");
		}
	else if(!strnicmp("m",instrg,1)) {
		sfprogstate.verbosemode = VERBOSE;
		if(DEBUGON) printf("Output mode is verbose\n");
		}
	else if(!strnicmp("q",instrg,1)) {
		sfprogstate.verbosemode = QUIET;
		if(DEBUGON) printf("Output mode is quiet\n");
		}
	else if(!strnicmp("h",instrg,1)) {
		DisplayHelp();
		}
	else if(!strnicmp("e",instrg,1)) {
		if(sfprogstate.logmode==YES)  {
			LogEvent("Stop log","");
			fclose(fplog);
			sfprogstate.logmode=NO;
			}
		SioDone(sfprogstate.comport);
		exit(1);
		}
	return returnstate;
}


void plot_datalogged_file(){

    FILE * acc_pipe = _popen("gnuplot -persist","w");
    FILE * gyr_pipe = _popen("gnuplot -persist","w");
    FILE * mag_pipe = _popen("gnuplot -persist","w");
    FILE * RPY_pipe1 = _popen("gnuplot -persist","w");
    FILE * RPY_pipe2 = _popen("gnuplot -persist","w");
    FILE * RPY_pipe3 = _popen("gnuplot -persist","w");


    fprintf(acc_pipe, "set term win\n");
    fprintf(acc_pipe, "set yrange [-3.000:3.000]\n");
    fprintf(acc_pipe, "set title 'Accelerometers'\n");
    fprintf(acc_pipe, "set xlabel 'Samples'\n");
    fprintf(acc_pipe, "set ylabel 'Amplitude (g)'\n");
    fprintf(acc_pipe, "set ytics 1 \n");
    fprintf(acc_pipe, "set mytics 10 \n");
    fprintf(acc_pipe, "set grid \n");
    fprintf(acc_pipe, "plot 'imu6410.log' using 3 with lines ti 'Accx' \n");
    fprintf(acc_pipe, "replot 'imu6410.log' using 4 with lines ti 'Accy' \n");
    fprintf(acc_pipe, "replot 'imu6410.log' using 5 with lines ti 'Accz'\n");
    fprintf(acc_pipe, "set term png\n");
    fprintf(acc_pipe, "set output \"acc.png\"\n");
    fprintf(acc_pipe, "replot\n");

    fprintf(gyr_pipe, "set win\n");
    fprintf(gyr_pipe, "set yrange [-300:300]\n");
    fprintf(gyr_pipe, "set title 'Gyroscope Data'\n");
    fprintf(gyr_pipe, "set xlabel 'Samples'\n");
    fprintf(gyr_pipe, "set ylabel 'Amplitude (deg/sec)'\n");
    fprintf(gyr_pipe, "set grid \n");
    fprintf(gyr_pipe, "plot 'imu6410.log' using 6 with lines ti 'Angular Velocity (x)' \n");
    fprintf(gyr_pipe, "replot 'imu6410.log' using 7 with lines ti 'Angular Velocity (y)' \n");
    fprintf(gyr_pipe, "replot 'imu6410.log' using 8 with lines ti 'Angular Velocity (z)'\n");
    fprintf(gyr_pipe, "set term png\n");
    fprintf(gyr_pipe, "set output \"gyr.png\"\n");
    fprintf(gyr_pipe, "replot\n");

    fprintf(gyr_pipe, "set term win\n");
    fprintf(gyr_pipe, "set yrange [-3:3]\n");
    fprintf(mag_pipe, "set yrange [-1:1]\n");
    fprintf(mag_pipe, "set title 'Magnetometer Data'\n");
    fprintf(mag_pipe, "set xlabel 'Samples'\n");
    fprintf(mag_pipe, "set ylabel 'Amplitude (deg/sec)'\n");
    fprintf(mag_pipe, "set grid \n");
    fprintf(mag_pipe, "plot 'imu6410.log' using 9 with lines ti 'Mag Field (x)' \n");
    fprintf(mag_pipe, "replot 'imu6410.log' using 10 with lines ti 'Mag Field (y)' \n");
    fprintf(mag_pipe, "replot 'imu6410.log' using 11 with lines ti 'Mag Field (z)'\n");
    fprintf(mag_pipe, "set term png\n");
    fprintf(mag_pipe, "set output \"mag.png\"\n");
    fprintf(mag_pipe, "replot\n");

    fprintf(RPY_pipe1, "set term win\n");
    fprintf(RPY_pipe1, "set yrange [-200:200]\n");
    fprintf(RPY_pipe1, "set title 'ROLL EKF PERFORMANCE'\n");
    fprintf(RPY_pipe1, "set xlabel 'Samples'\n");
    fprintf(RPY_pipe1, "set ylabel 'Angle (deg)'\n");
    fprintf(RPY_pipe1, "set grid \n");
    fprintf(RPY_pipe1, "plot 'imu6410.log' using 11 with lines ti 'Roll_KF' \n");
    fprintf(RPY_pipe1, "replot 'imu6410.log' using 14 with lines ti 'Roll_Acc' \n");
    fprintf(RPY_pipe1, "set term png\n");
    fprintf(RPY_pipe1, "set output \"RPY1.png\"\n");
    fprintf(RPY_pipe1, "replot\n");

    fprintf(RPY_pipe2, "set term win\n");
    fprintf(RPY_pipe2, "set yrange [-200:200]\n");
    fprintf(RPY_pipe2, "set title 'PITCH EKF PERFORMANCE'\n");
    fprintf(RPY_pipe2, "set xlabel 'Samples'\n");
    fprintf(RPY_pipe2, "set ylabel 'Angle (deg)'\n");
    fprintf(RPY_pipe2, "set grid \n");
    fprintf(RPY_pipe2, "plot 'datalog.dat' using 12 with lines ti 'Pitch_KF' \n");
    fprintf(RPY_pipe2, "replot 'datalog.dat' using 15 with lines ti 'Pitch_Acc' \n");
    fprintf(RPY_pipe2, "set term png\n");
    fprintf(RPY_pipe2, "set output \"RPY2.png\"\n");
    fprintf(RPY_pipe2, "replot\n");

    fprintf(RPY_pipe3, "set term win\n");
    fprintf(RPY_pipe3, "set yrange [-200:200]\n");
    fprintf(RPY_pipe3, "set title 'YAW EKF PERFORMANCE'\n");
    fprintf(RPY_pipe3, "set xlabel 'Samples'\n");
    fprintf(RPY_pipe3, "set ylabel 'Angle (deg)'\n");
    fprintf(RPY_pipe3, "set grid \n");
    fprintf(RPY_pipe3, "plot 'datalog.dat' using 13 with lines ti 'Yaw_KF' \n");
    fprintf(RPY_pipe3, "replot 'datalog.dat' using 16 with lines ti 'Yaw_Mag' \n");
    fprintf(RPY_pipe3, "set term png\n");
    fprintf(RPY_pipe3, "set output \"RPY3.png\"\n");
    fprintf(RPY_pipe3, "replot\n");

    _pclose(acc_pipe);
    _pclose(gyr_pipe);
    _pclose(mag_pipe);
    _pclose(RPY_pipe1);
    _pclose(RPY_pipe2);
    _pclose(RPY_pipe3);

}

int main(int argc, char* argv[])
{
	char tempstrg[200];
	char instrg[200],comfail;
	int  i,argptr,newcomchannel;

	SignonMessage();

	InitSfProg();

	// Recover from initial incorrect comchannel 
	//   correct channel should be in sfconfig.txt
	comfail = NO;
	while(InitSerialPort()==NOCOMCHANNEL)  {
		comfail = YES;
		printf("Enter new com channel [1,2,3,4,5,6,7,8]: ");
		scanf("%d",&sfprogstate.comport);
		sfprogstate.comport -= 1;
		}
 
	if(comfail==YES) {
		printf("\nThe default com channel can be set by creating a text file\n");
		printf("called sfconfig.txt in the current directory with the single\n");
		printf("line entry:\n\n");
	    printf("   comchannel comx\n\n");
		printf(" x is the COM port id\n\n");
		}

	if(argc>1) {
		argptr = 1;
		while(argptr<argc) {
			if(argv[argptr][0]=='-') {
				strcpy(tempstrg,argv[argptr]);
				for(i=0; i<strlen(tempstrg)-1; i++)  tempstrg[i] = tempstrg[i+1];
				tempstrg[i] = 0x00;
				ParseToken(tempstrg);
				}
			argptr++;
			}
		}
	else {
		printf("  Use COM%d,%d,N,1 - change using c or b commands\n",
			  sfprogstate.comport+1,ExpandBaudRate(sfprogstate.baudrate));
		printf("\n-");
		while(1) {
			scanf("%s",instrg);
			if(ParseToken(instrg)==DISPLAYBAR) printf("\n-");
			}
		}
		
//	printf("End of command line parse - type any character\n");
	//while(!kbhit())  ;
	SioDone(sfprogstate.comport);
	exit(1);

	printf("Plot IMU6410 EKF results and store in files\n");
	//plot_datalogged_file();
	return 0;
}

